"""Decorators for hubspot_api"""
import functools
from typing import Callable

from hubspot.crm.objects import ApiException
from rest_framework.status import HTTP_429_TOO_MANY_REQUESTS

from mitol.hubspot_api.exceptions import TooManyRequestsException


def raise_429(func) -> Callable:
    """
    Convert an ApiException to a TooManyRequestsException if status code is 429

     Returns:
         Callable: wrapped function (typically a celery task)
    """

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except ApiException as ae:
            if int(ae.status) == HTTP_429_TOO_MANY_REQUESTS:
                raise TooManyRequestsException(ae)
            else:
                raise

    return wrapper



"""Admin for hubspot CRM"""
from django.contrib import admin

from mitol.hubspot_api.models import HubspotObject


class HubspotObjectAdmin(admin.ModelAdmin):
    """Admin for HubspotObject"""

    model = HubspotObject
    list_display = ("content_object", "content_type", "object_id", "hubspot_id")
    list_filter = ("content_type",)

    def has_add_permission(self, request):
        return False


admin.site.register(HubspotObject, HubspotObjectAdmin)



""" Exceptions for hubspot_api"""
from hubspot.crm.objects import ApiException


class TooManyRequestsException(ApiException):
    """
    Exception to raise if Hubspot API returns a 429
    """

    pass



"""Factory for Hubspot API models"""
import string

import pytest
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from factory import Factory, Faker, LazyAttribute, Sequence, SubFactory
from factory.django import DjangoModelFactory
from factory.fuzzy import FuzzyInteger, FuzzyText
from hubspot.crm.objects import SimplePublicObject

from mitol.hubspot_api.models import HubspotObject

pytestmark = pytest.mark.django_db


class GroupFactory(DjangoModelFactory):
    """Factory for default Group model"""

    name = FuzzyText()

    class Meta:
        model = Group


class HubspotObjectFactory(DjangoModelFactory):
    """Factory for HubspotObjects"""

    content_object = SubFactory(GroupFactory)
    content_type = LazyAttribute(
        lambda o: ContentType.objects.get_for_model(o.content_object)
    )
    object_id = FuzzyInteger(1, 999999999)
    hubspot_id = FuzzyText(chars=string.digits)

    class Meta:
        model = HubspotObject


class SimplePublicObjectFactory(Factory):
    """Factory for SimplePublicObject"""

    id = Sequence(lambda number: "111000{}".format(number))
    properties = Faker("pydict")

    class Meta:
        model = SimplePublicObject



"""
Hubspot settings
"""

from mitol.common.envs import get_int, get_string

MITOL_HUBSPOT_API_PRIVATE_TOKEN = get_string(
    name="MITOL_HUBSPOT_API_PRIVATE_TOKEN",
    default=None,
    description="Private token for Hubspot",
)
MITOL_HUBSPOT_API_ID_PREFIX = get_string(
    name="MITOL_HUBSPOT_API_ID_PREFIX",
    default="app",
    description="Prefix for generating custom hubspot_api ids",
)
MITOL_HUBSPOT_API_RETRIES = get_int(
    name="MITOL_HUBSPOT_API_RETRIES",
    default=3,
    description="Number of times to retry failed API requests",
)




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/mitol/hubspot_api:mitol-hubspot-api

from setuptools import setup

setup(**{
    'authors': [
        'MIT Office of Open Learning <mitx-devops@mit.edu>',
    ],
    'classifiers': [
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    'description': 'Django application for Hubspot API integration',
    'install_requires': (
        'django<4.0,>=2.2.12',
        'djangorestframework>=3.0.0',
        'factory-boy~=3.2',
        'hubspot-api-client==6.1.0',
        'mitol-django-common~=2.7.0',
        'pytest==6.1.2',
        'setuptools',
        'urllib3>=1.26.5',
    ),
    'license': 'BSD 3-Clause License',
    'long_description': """mitol-django-hubspot-api
---

This is the Open Learning Hubspot API integration app. It provides helper functions for Hubspot CRM API calls:

- CRUD functions for custom properties and property groups 
- CRUD functions for deals, line items, products, and contacts
- Search/retrieve specific objects or lists of objects of a certain type

### Getting started

`pip install mitol-django-hubspot-api`

Add the hubspot app:

```python
INSTALLED_APPS = [
    ...
    "mitol.hubspot_api.apps.HubspotApp",
]
```

### Settings

#### Hubspot app settings

All settings for the `mitol-django-hubspot-api` app are namespaced in django settings with `MITOL_HUBSPOT_API` prefix.

- `MITOL_HUBSPOT_API_PRIVATE_TOKEN` - the private app token to be used for authentication (required)
- `MITOL_HUBSPOT_API_RETRIES` - the number of times to retry API calls on failures (default=3)
- `MITOL_HUBSPOT_API_ID_PREFIX` - a prefix used for generating custom unique object ids (default="app")

### Usage

#### Instantiate an API client to make custom hubspot requests

```python
from mitol.hubspot_api.api import HubspotApi, HubspotObjectType

client = HubspotApi()
client.crm.objects.basic_api.update(
    simple_public_object_input=input_body,
    object_id=123,
    object_type=HubspotObjectType.DEALS.value,
)
```

#### Use helper functions to make common hubspot requests
```python
from mitol.hubspot_api.api import find_product

hubspot_product = find_product("Product #1", price="123.99")
```
""",
    'long_description_content_type': 'text/markdown',
    'name': 'mitol-django-hubspot-api',
    'namespace_packages': (
        'mitol',
    ),
    'package_data': {
        'mitol.hubspot_api': (
            'CHANGELOG.md',
            'README.md',
            'py.typed',
        ),
    },
    'packages': (
        'mitol',
        'mitol.hubspot_api',
        'mitol.hubspot_api.migrations',
        'mitol.hubspot_api.settings',
    ),
    'python_requires': '>=3.7',
    'version': '1.1.0',
    'zip_safe': True,
})



# Generated by Django 3.2.15 on 2022-10-13 20:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="HubspotObject",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("hubspot_id", models.CharField(max_length=255)),
                ("object_id", models.PositiveIntegerField()),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
        ),
        migrations.AddIndex(
            model_name="hubspotobject",
            index=models.Index(
                fields=["content_type", "object_id"],
                name="hubspot_api_content_object_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="hubspotobject",
            constraint=models.UniqueConstraint(
                fields=("object_id", "content_type"),
                name="hubspot_api_unique_object_id_type",
            ),
        ),
        migrations.AddConstraint(
            model_name="hubspotobject",
            constraint=models.UniqueConstraint(
                fields=("hubspot_id", "content_type"),
                name="hubspot_api_unique_hubspot_id_type",
            ),
        ),
    ]
